@@ -29,6 +29,7 @@ SLcurses_Window_Type *SLcurses_Stdscr;
 int SLcurses_Esc_Delay = 150;	       /* 0.15 seconds */
 SLtt_Char_Type SLcurses_Acs_Map [128];
 int SLcurses_Is_Endwin = 1;
+int SLcurses_Num_Colors = 8;
 
 static void blank_line (unsigned short *b, unsigned int len, unsigned short color)
 {
@@ -40,33 +41,60 @@ static void blank_line (unsigned short *b, unsigned int len, unsigned short colo
    while (b < bmax) *b++ = color;
 }
 
+static int va_mvprintw (SLcurses_Window_Type *w, int r, int c, int do_move,
+			char *fmt, va_list ap)
+{
+   char buf[1024];
+   
+   if (do_move) SLcurses_wmove (w, r, c);
+
+   (void) _SLvsnprintf (buf, sizeof(buf), fmt, ap);
+
+   SLcurses_waddnstr (w, buf, -1);
+   return 0;
+}
+
 int SLcurses_mvprintw (int r, int c, char *fmt, ...)
 {
-   char p[1000];
    va_list ap;
 
-   SLcurses_wmove (SLcurses_Stdscr, r, c);
-
    va_start(ap, fmt);
-   (void) vsprintf(p, fmt, ap);
+   va_mvprintw (SLcurses_Stdscr, r, c, 1, fmt, ap);
    va_end(ap);
 
-   SLcurses_waddnstr (SLcurses_Stdscr, p, -1);
    return 0;
 }
 
 int SLcurses_mvwprintw (SLcurses_Window_Type *w, int r, int c, char *fmt, ...)
 {
-   char p[1000];
    va_list ap;
 
-   SLcurses_wmove (w, r, c);
-
    va_start(ap, fmt);
-   (void) vsprintf(p, fmt, ap);
+   va_mvprintw (w, r, c, 1, fmt, ap);
+   va_end(ap);
+
+   return 0;
+}
+
+int SLcurses_wprintw (SLcurses_Window_Type *w, char *fmt, ...)
+{
+   va_list ap;
+
+   va_start(ap, fmt);
+   va_mvprintw (w, 0, 0, 0, fmt, ap);
+   va_end(ap);
+
+   return 0;
+}
+
+int SLcurses_printw (char *fmt, ...)
+{
+   va_list ap;
+
+   va_start(ap, fmt);
+   va_mvprintw (SLcurses_Stdscr, 0, 0, 0, fmt, ap);
    va_end(ap);
 
-   SLcurses_waddnstr (w, p, -1);
    return 0;
 }
 
@@ -159,9 +187,9 @@ int SLcurses_start_color (void)
    if (SLtt_Use_Ansi_Colors == 0) return -1;
 
    obj = 0;
-   for (f = 0; f < 8; f++)
+   for (f = 0; f < 16; f++)
      {
-	for (b = 0; b < 8; b++)
+	for (b = 0; b < 16; b++)
 	  {
 	     obj++;
 	     SLtt_set_color_fgbg (obj, f, b);
@@ -229,9 +257,12 @@ static void sigtstp_handler (int sig)
 SLcurses_Window_Type *SLcurses_initscr (void)
 {
    SLcurses_Is_Endwin = 0;
-   SLsmg_Newline_Moves = -1;
+   SLsmg_Newline_Behavior = SLSMG_NEWLINE_MOVES;
    SLtt_get_terminfo ();
 
+   if (-1 == (SLcurses_Num_Colors = SLtt_tgetnum ("Co")))
+     SLcurses_Num_Colors = 8;
+
    if ((-1 == SLkp_init ())
        || (-1 == SLcurses_cbreak ())
        || (NULL == (SLcurses_Stdscr = SLcurses_newwin (0, 0, 0, 0)))
@@ -733,18 +764,6 @@ int SLcurses_waddnstr (SLcurses_Window_Type *w, char *str, int len)
    return 0;
 }
 
-int SLcurses_wprintw (SLcurses_Window_Type *w, char *fmt, ...)
-{
-   char p[1000];
-   va_list ap;
-
-   va_start(ap, fmt);
-   (void) vsprintf(p, fmt, ap);
-   va_end(ap);
-
-   SLcurses_waddnstr (w, p, -1);
-   return 0;
-}
 
 /* This routine IS NOT CORRECT.  It needs to compute the proper overlap
  * and copy accordingly.  Here, I just assume windows are same size.
@@ -837,18 +856,6 @@ int SLcurses_wclear (SLcurses_Window_Type *w)
    return 0;
 }
 
-int SLcurses_printw (char *fmt, ...)
-{
-   char p[1000];
-   va_list ap;
-
-   va_start(ap, fmt);
-   (void) vsprintf(p, fmt, ap);
-   va_end(ap);
-
-   SLcurses_waddnstr (SLcurses_Stdscr, p, -1);
-   return 0;
-}
 
 int SLcurses_wdelch (SLcurses_Window_Type *w)
 {
