@@ -1,15 +1,15 @@
-/* Copyright (c) 1992, 1995 John E. Davis
- * All rights reserved.
- * 
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of the S-Lang library.
+ *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
 
 #include "config.h"
+#include "sl-feat.h"
 
 #include <stdio.h>
 
-
 #include <dos.h>
 
 #if defined (__EMX__)
@@ -29,7 +29,7 @@
 #define BIOSKEY		slbioskey
 #if defined(__WATCOMC__)
 # define keyWaiting() _bios_keybrd(_NKEYBRD_READY)
-#else 
+#else
 # define keyWaiting()	BIOSKEY(_NKEYBRD_READY)
 #endif
 
@@ -68,7 +68,7 @@ static void interrupt int9_handler (_DOTS_)
      {
 	if (s == Abort_Scan_Code)
 	  {
-	     if (SLang_Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
+	     if (SLang_Ignore_User_Abort == 0) SLang_Error = SL_USER_BREAK;
 	     SLKeyBoard_Quit = 1;
 	  }
      }
@@ -107,7 +107,7 @@ static void set_ctrl_break (int state)
 #else	/* __EMX__ */
 
    static int prev = 0;
-   
+
 # if defined (__GO32__)
    if (state == 0)
      {
@@ -231,26 +231,32 @@ void SLang_reset_tty (void)
 }
 
 /*----------------------------------------------------------------------*\
- *  Function:	int SLsys_input_pending (int tsecs);
+ *  Function:	int _SLsys_input_pending (int tsecs);
  *
  *  sleep for *tsecs tenths of a sec waiting for input
 \*----------------------------------------------------------------------*/
-int SLsys_input_pending (int tsecs)
+int _SLsys_input_pending (int tsecs)
 {
-   int count = tsecs * 5;
-
    if (keyWaiting()) return 1;
-   while (count > 0)
+   
+   /* Convert tsecs to units of 20 ms */
+   tsecs = tsecs * 5;
+
+   /* If tsecs is less than 0, it represents millisecs */
+   if (tsecs < 0)
+     tsecs = -tsecs / 100;
+	
+   while (tsecs > 0)
      {
 	delay (20);	/* 20 ms or 1/50 sec */
 	if (keyWaiting()) break;
-	count--;
+	tsecs--;
      }
-   return (count);
+   return (tsecs);
 }
 
 /*----------------------------------------------------------------------*\
- *  Function:	unsigned int SLsys_getkey (void);
+ *  Function:	unsigned int _SLsys_getkey (void);
  *
  * wait for and get the next available keystroke.
  * Also re-maps some useful keystrokes.
@@ -259,7 +265,7 @@ int SLsys_input_pending (int tsecs)
  *	Ctrl-Space	=>	^@	(^@^3 - a pc NUL char)
  *	extended keys are prefixed by a null character
 \*----------------------------------------------------------------------*/
-unsigned int SLsys_getkey (void)
+unsigned int _SLsys_getkey (void)
 {
    unsigned int key, scan, ch, shift;
 
@@ -267,7 +273,7 @@ unsigned int SLsys_getkey (void)
    int tsecs = SL_input_time;	/* tsecs = 300 */
    if ((SLgetkey_hook != NULL) && !keyWaiting())
      {
-	while (!SLsys_input_pending(tsecs))
+	while (!_SLsys_input_pending(tsecs))
 	  getkey_hook ();
      }
 #endif
@@ -277,17 +283,20 @@ unsigned int SLsys_getkey (void)
    scan = key >> 8;
    shift = BIOSKEY(_NKEYBRD_SHIFTSTATUS) & 0xf;
 
-   if (key == 0x0e08) ch = 127;			/* Backspace key */
-   else if ((ch == 32) && (shift & 0x04))	/* ^space = ^@ */
+   if (key == 0x0e08) 
+     return 127;			/* Backspace key */
+   
+   switch (ch)
      {
+      case 32:
+	if (0 == (shift & 0x04))
+	  break;
+	/* ^space = ^@ */
 	scan = 3;		/* send back Ctrl-@ => ^@^C */
-	ch = 0;
-	SLang_ungetkey (scan);
-     }
-   else if (!(ch) || (ch == 0xe0))
-     {
-	ch = 0;			/* extended key code */
-	SLang_ungetkey (scan);
+	/* drop */
+      case 0xe0:
+      case 0:			/* extended key code */
+	ch = _SLpc_convert_scancode (scan);
      }
    return (ch);
 }
@@ -295,7 +304,17 @@ unsigned int SLsys_getkey (void)
 /*----------------------------------------------------------------------*\
  *  Function:	void SLang_set_abort_signal (void (*handler)(int));
 \*----------------------------------------------------------------------*/
-void SLang_set_abort_signal (void (*handler)(int))
+int SLang_set_abort_signal (void (*handler)(int))
 {
    if (handler == NULL) int9_change (1);
+   return 0;
+}
+
+int SLtt_set_mouse_mode (int mode, int force)
+{
+   /* FIXME !!! */
+   (void) mode;
+   (void) force;
+   
+   return -1;
 }
