@@ -67,29 +67,48 @@ static char *get_error_string (void)
 
 void SLang_doerror (char *error)
 {
-   char err_buf [1024];
    char *str = NULL;
-   char *sle = "S-Lang Error: ";
    char *err;
+   char *malloced_err_buf;
+   char err_buf [1024];
 
-   err = err_buf;
-   *err = 0;
+   malloced_err_buf = NULL;
 
    if ((SLang_Error == SL_USER_ERROR)
        && (error != NULL) && (*error != 0))
      err = error;
    else
      {
+	char *sle = "S-Lang Error: ";
+	unsigned int len;
+	char *fmt;
+
 	str = get_error_string ();
 
+	fmt = "%s%s%s";
 	if ((error == NULL) || (*error == 0))
-	  sprintf (err, "%s%s", sle, str);
+	  error = "";
 	else if (SLang_Error == SL_UNKNOWN_ERROR)
 	  /* Do not display an unknown error message if error is non-NULL */
-	  sprintf (err, "%s%s", sle, error);
+	  str = "";
 	else
-	  sprintf (err, "%s%s: %s", sle, str, error);
+	  fmt = "%s%s: %s";
+	
+	len = strlen (sle) + strlen (str) + strlen(error) + 1;
+
+	err = err_buf;
+	if (len >= sizeof (err_buf))
+	  {
+	     if (NULL == (malloced_err_buf = SLmalloc (len)))
+	       err = NULL;
+	     else
+	       err = malloced_err_buf;
+	  }
+
+	if (err != NULL) sprintf (err, fmt, sle, str, error);
+	else err = "Out of memory";
      }
+   
 
    if (SLang_Error_Hook == NULL)
      {
@@ -98,6 +117,8 @@ void SLang_doerror (char *error)
      }
    else
      (*SLang_Error_Hook)(err);
+
+   SLfree (malloced_err_buf);
 }
 
 void SLang_verror (int err_code, char *fmt, ...)
@@ -108,18 +129,12 @@ void SLang_verror (int err_code, char *fmt, ...)
    if (err_code == 0) err_code = SL_INTRINSIC_ERROR;
    if (SLang_Error == 0) SLang_Error = err_code;
 
-   /* I would like to use vsnprintf but I do not know how common it
-    * is.
-    */
-   if (4 * strlen (fmt) > 3 * sizeof (err))
-     fmt = NULL;
-
    if (fmt != NULL)
      {
 	va_start(ap, fmt);
-	(void) vsprintf (err, fmt, ap);
-	va_end(ap);
+	(void) _SLvsnprintf (err, sizeof (err), fmt, ap);
 	fmt = err;
+	va_end(ap);
      }
 
    SLang_doerror (fmt);
