@@ -49,17 +49,19 @@ static SLang_Key_Type *malloc_key(unsigned char *str)
    return(neew);
 }
 
-static SLKeyMap_List_Type *add_keymap(char *name, SLang_Key_Type *map)
+static SLKeyMap_List_Type *add_keymap (char *name, SLang_Key_Type *map)
 {
    int i;
-
+   
    for (i = 0; i < SLANG_MAX_KEYMAPS; i++)
      {
 	if (SLKeyMap_List[i].keymap == NULL)
 	  {
+	     if (NULL == (name = SLang_create_slstring (name)))
+	       return NULL;
+
 	     SLKeyMap_List[i].keymap = map;
-	     strncpy((char *) SLKeyMap_List[i].name, (char *) name, MAX_KEYMAP_NAME_LEN);
-	     SLKeyMap_List[i].name[MAX_KEYMAP_NAME_LEN] = 0;
+	     SLKeyMap_List[i].name = name;
 	     return &SLKeyMap_List[i];
 	  }
      }
@@ -71,16 +73,17 @@ static SLKeyMap_List_Type *add_keymap(char *name, SLang_Key_Type *map)
 FVOID_STAR SLang_find_key_function(char *name, SLKeyMap_List_Type *keymap)
 {
    SLKeymap_Function_Type *fp = keymap -> functions;
+   char ch = *name;
 
    while ((fp != NULL) && (fp->name != NULL))
      {
-	if ((*fp->name == *name) && !strcmp(fp->name, name))
-	  {
-	     return (FVOID_STAR) fp->f;
-	  }
+	if ((ch == *fp->name)
+	    && (0 == strcmp(fp->name, name)))
+	  return (FVOID_STAR) fp->f;
+
 	fp++;
      }
-   return(NULL);
+   return NULL;
 }
 
 #ifdef REAL_UNIX_SYSTEM
@@ -158,6 +161,13 @@ char *SLang_process_keystring(char *s)
 	if (i >= 32) break;
 	str[i++] = ch;
      }
+   
+   if (i > SLANG_MAX_KEYMAP_KEY_SEQ)
+     {
+	SLang_verror (SL_INVALID_PARM, "Key sequence is too long");
+	return NULL;
+     }
+
    str[0] = i;
    return(str);
 }
@@ -200,7 +210,8 @@ static int find_the_key (char *s, SLKeyMap_List_Type *kml, SLang_Key_Type **keyp
 
    *keyp = NULL;
 
-   str = (unsigned char *) SLang_process_keystring(s);
+   if (NULL == (str = (unsigned char *) SLang_process_keystring(s)))
+     return -2;
 
    if (1 == (str_len = str[0]))
      return 0;
@@ -457,7 +468,8 @@ void SLang_undefine_key(char *s, SLKeyMap_List_Type *kml)
    unsigned char *str;
 
    keymap = kml -> keymap;
-   str = (unsigned char *) SLang_process_keystring(s);
+   if (NULL == (str = (unsigned char *) SLang_process_keystring(s)))
+     return;
 
    if (0 == (n = *str++ - 1)) return;
    i = *str;
@@ -490,10 +502,16 @@ void SLang_undefine_key(char *s, SLKeyMap_List_Type *kml)
 
 char *SLang_make_keystring(unsigned char *s)
 {
-   static char buf[40];
+   static char buf [3 * SLANG_MAX_KEYMAP_KEY_SEQ];
    char *b;
    int n;
 
+   if (strlen ((char *)s) > SLANG_MAX_KEYMAP_KEY_SEQ)
+     {
+	SLang_verror (SL_INVALID_PARM, "Key sequence is too long");
+	return NULL;
+     }
+
    b = buf;
    n = *s++ - 1;
    while (n--)
@@ -568,11 +586,18 @@ SLKeyMap_List_Type *SLang_create_keymap(char *name, SLKeyMap_List_Type *map)
 
 SLKeyMap_List_Type *SLang_find_keymap(char *name)
 {
-   int i;
-
-   for (i = 0; i < SLANG_MAX_KEYMAPS; i++)
+   SLKeyMap_List_Type *kmap, *kmap_max;
+   
+   kmap = SLKeyMap_List;
+   kmap_max = kmap + SLANG_MAX_KEYMAPS;
+   
+   while (kmap < kmap_max)
      {
-	if (!strncmp(SLKeyMap_List[i].name, name, 8)) return &SLKeyMap_List[i];
+	if ((kmap->name != NULL)
+	    && (0 == strcmp (kmap->name, name)))
+	  return kmap;
+	
+	kmap++;
      }
-   return(NULL);
+   return NULL;
 }
