@@ -1,19 +1,19 @@
 /* ed style regular expressions */
-/* Copyright (c) 1992, 1995 John E. Davis
- * All rights reserved.
- * 
+/* Copyright (c) 1992, 1998 John E. Davis
+ *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
 
-
-#include <config.h>
+#include "config.h"
+#include "sl-feat.h"
 
 #include <stdio.h>
 #include <string.h>
 
-
 #include "slang.h"
+#include "_slang.h"
+#include "jdmacros.h"
 
 #define SET_BIT(b, n) b[(unsigned int) (n) >> 3] |= 1 << ((unsigned int) (n) % 8)
 #define TEST_BIT(b, n) (b[(unsigned int)(n) >> 3] & (1 << ((unsigned int) (n) % 8)))
@@ -45,37 +45,41 @@
 static unsigned char Word_Chars[256];
 #define IS_WORD_CHAR(x) Word_Chars[(unsigned int) (x)]
 
-static int Next_Pos;
+static int Open_Paren_Number;
+static char Closed_Paren_Matches[10];
+
 static SLRegexp_Type *This_Reg;
 static unsigned char *This_Str;
 
 static unsigned char *do_nth_match(int n, unsigned char *str, unsigned char *estr)
 {
    unsigned char *bpos;
-   if (n > Next_Pos) return (NULL);
-   
+
+   if (Closed_Paren_Matches[n] == 0)
+     return NULL;
+
    bpos = This_Reg->beg_matches[n] + This_Str;
    n = This_Reg->end_matches[n];
    if (n == 0) return(str);
    if (n > (int) (estr - str)) return (NULL);
-   
+
    /* This needs fixed for case sensitive match */
    if (0 != strncmp((char *) str, (char *) bpos, (unsigned int) n)) return (NULL);
    str += n;
    return (str);
 }
 
-
 /* returns pointer to the end of regexp or NULL */
 static unsigned char *regexp_looking_at(register unsigned char *str, unsigned char *estr, unsigned char *buf, register int cs)
 {
    register unsigned char p, p1;
    unsigned char *save_str, *tmpstr;
    int n, n0, n1;
-   int save_next = 0;
+   int save_num_open;
+   char save_closed_matches[10];
+
    p = *buf++;
-   
-   
+
    while (p != 0)
      {
 	/* p1 = UPPERCASE(*buf); */
@@ -89,29 +93,39 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 		     || IS_WORD_CHAR(*(str - 1))
 		     || (0 == IS_WORD_CHAR(*str)))) return NULL;
 	     break;
-	     
+
 	   case EOW:
 	     if ((str < estr)
 		 && IS_WORD_CHAR (*str)) return NULL;
 	     break;
-	     
+
 	   case YES_CASE: cs = 1; break;
 	   case NO_CASE: cs = 0; break;
 
 	   case OPAREN:
-	     This_Reg->beg_matches[Next_Pos] = (int) (str - This_Str);
-	     This_Reg->beg_matches[Next_Pos + 1] = -1;
+	     Open_Paren_Number++;
+	     This_Reg->beg_matches[Open_Paren_Number] = (int) (str - This_Str);
 	     break;
-	   case CPAREN: 
-	     This_Reg->end_matches[Next_Pos] = (unsigned int) (str - This_Str) - This_Reg->beg_matches[Next_Pos];
-	     Next_Pos++;
+	   case CPAREN:
+	     n = Open_Paren_Number;
+	     while (n > 0)
+	       {
+		  if (Closed_Paren_Matches[n] != 0)
+		    {
+		       n--;
+		       continue;
+		    }
+		  Closed_Paren_Matches[n] = 1;
+		  This_Reg->end_matches[n] = (unsigned int) (str - (This_Str + This_Reg->beg_matches[n]));
+		  break;
+	       }
 	     break;
 #ifdef NOT_LITERAL
 	   case NOT_LITERAL:
 	     if ((str >= estr) || (*buf == UPPERCASE(*str))) return (NULL);
 	     str++; buf++;
 	     break;
-	     
+
 	   case MAYBE_ONCE | NOT_LITERAL:
 	     save_str = str;
 	     if ((str < estr) && (*buf != UPPERCASE(*str))) str++;
@@ -127,7 +141,7 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     while ((str < estr) && (UPPERCASE(*str) != p1)) str++;
 	     buf++;
 	     goto match_rest;
-	     
+
 	     /* this type consists of the expression + two bytes that
 	        determine number of matches to perform */
 	   case MANY | NOT_LITERAL:
@@ -137,27 +151,27 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     n1 = (int) (unsigned char) *buf++;
 	     /* maximum number to match */
 
-	     while (n && (str < estr) && (p1 != *str)) 
+	     while (n && (str < estr) && (p1 != *str))
 	       {
 		  n--;
 		  str++;
 	       }
 	     if (n) return (NULL);
-	     
+
 	     save_str = str;
 	     n = n1 - n0;
-	     while (n && (str < estr) && (p1 != *str)) 
+	     while (n && (str < estr) && (p1 != *str))
 	       {
 		  n--;
 		  str++;
 	       }
 	     goto match_rest;
 #endif				       /* NOT_LITERAL */
-	   case LITERAL: 
+	   case LITERAL:
 	     if ((str >= estr) || (*buf != UPPERCASE(*str))) return (NULL);
 	     str++; buf++;
 	     break;
-	     
+
 	   case MAYBE_ONCE | LITERAL:
 	     save_str = str;
 	     if ((str < estr) && (*buf == UPPERCASE(*str))) str++;
@@ -173,7 +187,7 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     while ((str < estr) && (UPPERCASE(*str) == p1)) str++;
 	     buf++;
 	     goto match_rest;
-	     
+
 	     /* this type consists of the expression + two bytes that
 	        determine number of matches to perform */
 	   case MANY | LITERAL:
@@ -183,27 +197,27 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     n1 = (int) (unsigned char) *buf++;
 	     /* maximum number to match */
 
-	     while (n && (str < estr) && (p1 == *str)) 
+	     while (n && (str < estr) && (p1 == *str))
 	       {
 		  n--;
 		  str++;
 	       }
 	     if (n) return (NULL);
-	     
+
 	     save_str = str;
 	     n = n1 - n0;
-	     while (n && (str < estr) && (p1 == *str)) 
+	     while (n && (str < estr) && (p1 == *str))
 	       {
 		  n--;
 		  str++;
 	       }
 	     goto match_rest;
-	     
-	   case NTH_MATCH: 
+
+	   case NTH_MATCH:
 	     if ((str = do_nth_match((int) (unsigned char) *buf, str, estr)) == NULL) return(NULL);
 	     buf++;
 	     break;
-	     
+
 	   case MAYBE_ONCE | NTH_MATCH:
 	     save_str = str;
 	     tmpstr = do_nth_match((int) (unsigned char) *buf, str, estr);
@@ -218,7 +232,7 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	   case LEAST_ONCE | NTH_MATCH:
 	     if ((str = do_nth_match((int) (unsigned char) *buf, str, estr)) == NULL) return(NULL);
 	     /* drop */
-	   case STAR | NTH_MATCH: 
+	   case STAR | NTH_MATCH:
 	     save_str = str;
 	     while (NULL != (tmpstr = do_nth_match((int) (unsigned char) *buf, str, estr)))
 	       {
@@ -226,23 +240,23 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	       }
 	     buf++;
 	     goto match_rest;
-	     
+
 	   case MANY | NTH_MATCH: return(NULL);
 	     /* needs done */
-	     
-	   case RANGE: 
+
+	   case RANGE:
 	     if (str >= estr) return (NULL);
 	     if (TEST_BIT(buf, UPPERCASE(*str)) == 0) return (NULL);
 	     buf += 32; str++;
 	     break;
-	     
+
 	   case MAYBE_ONCE | RANGE:
 	     save_str = str;
 	     if ((str < estr) && TEST_BIT(buf, UPPERCASE(*str))) str++;
 	     buf += 32;
 	     goto match_rest;
 
-	   case LEAST_ONCE | RANGE: 
+	   case LEAST_ONCE | RANGE:
 	     if ((str >= estr) || (0 == TEST_BIT(buf, UPPERCASE(*str)))) return NULL;
 	     str++;
 	     /* drop */
@@ -251,11 +265,11 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     while ((str < estr) && TEST_BIT(buf, UPPERCASE(*str))) str++;
 	     buf += 32;
 	     goto match_rest;
-	     
-	     /* The first 32 bytes correspond to the range and the two 
+
+	     /* The first 32 bytes correspond to the range and the two
 	      * following bytes indicate the min and max number of matches.
 	      */
-	   case MANY | RANGE:  
+	   case MANY | RANGE:
 	     /* minimum number to match--- could be 0 */
 	     n = n0 = (int) (unsigned char) *(buf + 32);
 	     /* maximum number to match */
@@ -276,7 +290,7 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	       }
 	     buf += 34;		       /* 32 + 2 */
 	     goto match_rest;
-	     
+
 	   case ANY_DIGIT:
 	     if ((str >= estr) || (*str > '9') || (*str < '0')) return (NULL);
 	     str++;
@@ -295,16 +309,16 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     save_str = str;
 	     while ((str < estr) && ((*str <= '9') && (*str >= '0'))) str++;
 	     goto match_rest;
-	     
-	   case MANY | ANY_DIGIT: 
+
+	   case MANY | ANY_DIGIT:
 	     /* needs finished */
 	     return (NULL);
-	     
+
 	   case ANY:
 	     if ((str >= estr) || (*str == '\n')) return (NULL);
 	     str++;
 	     break;
-	     
+
 	   case MAYBE_ONCE | ANY:
 	     save_str = str;
 	     if ((str < estr) && (*str != '\n')) str++;
@@ -318,12 +332,12 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	     save_str = str;
 	     while ((str < estr) && (*str != '\n')) str++;
 	     goto match_rest;
-	     
-	   case MANY | ANY: 
+
+	   case MANY | ANY:
 	     return (NULL);
 	     /* needs finished */
 
-	   case EOL: 
+	   case EOL:
 	     if ((str >= estr) || (*str == '\n')) return (str);
 	     return(NULL);
 
@@ -331,66 +345,96 @@ static unsigned char *regexp_looking_at(register unsigned char *str, unsigned ch
 	  }
 	p = *buf++;
 	continue;
-	
+
 	match_rest:
 	if (save_str == str)
 	  {
 	     p = *buf++;
 	     continue;
 	  }
-	
+
 	/* if (p == EOL)
 	 * {
 	 * if (str < estr) return (NULL); else return (str);
-	 * } 
+	 * }
 	 */
-	
-	save_next = Next_Pos;
+
+	SLMEMCPY(save_closed_matches, Closed_Paren_Matches, sizeof(save_closed_matches));
+	save_num_open = Open_Paren_Number;
 	while (str >= save_str)
 	  {
 	     tmpstr = regexp_looking_at(str, estr, buf, cs);
 	     if (tmpstr != NULL) return(tmpstr);
-	     Next_Pos = save_next;
+	     SLMEMCPY(Closed_Paren_Matches, save_closed_matches, sizeof(Closed_Paren_Matches));
+	     Open_Paren_Number = save_num_open;
 	     str--;
 	  }
-	return (NULL);
+	return NULL;
      }
    if ((p != 0) && (p != EOL)) return (NULL); else return (str);
 }
 
+static void
+fixup_beg_end_matches (SLRegexp_Type *r, unsigned char *str, unsigned char *epos)
+{
+   int i;
 
-unsigned char *SLang_regexp_match(unsigned char *str, 
+   if (str == NULL)
+     {
+	r->beg_matches[0] = -1;
+	r->end_matches[0] = 0;
+	SLMEMSET(Closed_Paren_Matches, 0, sizeof(Closed_Paren_Matches));
+     }
+   else
+     {
+	r->beg_matches[0] = (int) (str - This_Str);
+	r->end_matches[0] = (unsigned int) (epos - str);
+     }
+
+   for (i = 1; i < 10; i++)
+     {
+	if (Closed_Paren_Matches [i] == 0)
+	  {
+	     r->beg_matches[i] = -1;
+	     r->end_matches[i] = 0;
+	  }
+     }
+}
+
+unsigned char *SLang_regexp_match(unsigned char *str,
 				  unsigned int len, SLRegexp_Type *reg)
 {
    register unsigned char c = 0, *estr = str + len;
    int cs = reg->case_sensitive, lit = 0;
-   unsigned char *buf = reg->buf, *epos;
+   unsigned char *buf = reg->buf, *epos = NULL;
 
    if (reg->min_length > len) return NULL;
-   
-   Next_Pos = 1;
+
+   Open_Paren_Number = 0;
+   SLMEMSET(Closed_Paren_Matches, 0, sizeof(Closed_Paren_Matches));
    This_Reg = reg;
    This_Str = str;
 
-   if (*buf == BOL) 
+   if (*buf == BOL)
      {
-	if (NULL == (epos = regexp_looking_at(str, estr, buf + 1, cs))) str = NULL;
-	reg->beg_matches[0] = (int) (str - This_Str);
-	reg->end_matches[0] = (unsigned int) (epos - str);
-	return(str);
+	if (NULL == (epos = regexp_looking_at(str, estr, buf + 1, cs)))
+	  str = NULL;
+
+	fixup_beg_end_matches (reg, str, epos);
+	return str;
      }
 
    if (*buf == NO_CASE)
      {
 	buf++;  cs = 0;
      }
-   
+
    if (*buf == YES_CASE)
      {
 	buf++;  cs = 1;
      }
-   
-   if (*buf == LITERAL) 
+
+   if (*buf == LITERAL)
      {
 	lit = 1;
 	c = *(buf + 1);
@@ -400,25 +444,28 @@ unsigned char *SLang_regexp_match(unsigned char *str,
 	lit = 1;
 	c = *(buf + 2);
      }
-   
+
    while (str < estr)
      {
+	Open_Paren_Number = 0;
+	SLMEMSET(Closed_Paren_Matches, 0, sizeof(Closed_Paren_Matches));
 	/* take care of leading chars */
 	if (lit)
 	  {
 	     while ((str < estr) && (c != UPPERCASE(*str))) str++;
-	     if (str >= estr) return(NULL);
+	     if (str >= estr)
+	       break;		       /* failed */
 	  }
-	Next_Pos = 1;
+
 	if (NULL != (epos = regexp_looking_at(str, estr, buf, cs)))
 	  {
-	     reg->beg_matches[0] = (int) (str - This_Str);
-	     reg->end_matches[0] = (unsigned int) (epos - str);
-	     return (str);
+	     fixup_beg_end_matches (reg, str, epos);
+	     return str;
 	  }
 	str++;
      }
-   return (NULL);
+   fixup_beg_end_matches (reg, NULL, epos);
+   return NULL;
 }
 
 static unsigned char *convert_digit(unsigned char *pat, int *nn)
@@ -431,7 +478,7 @@ static unsigned char *convert_digit(unsigned char *pat, int *nn)
 	n = 10 * n + (c - '0');
 	m++;
      }
-   if (m == 0) 
+   if (m == 0)
      {
 	return (NULL);
      }
@@ -448,13 +495,13 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    unsigned char *last = NULL, *tmppat;
    register unsigned char c;
    int i, reverse = 0, n, cs;
-   int oparen = 0, dparen = 0, nparen = 0;
+   int oparen = 0, nparen = 0;
    /* substring stuff */
-   int count, last_count, this_max_mm = 0, max_mm = 0, ordinary_search, 
+   int count, last_count, this_max_mm = 0, max_mm = 0, ordinary_search,
      no_osearch = 0, min_length = 0;
    unsigned char *mm_p = NULL, *this_mm_p = NULL;
    static int already_initialized;
-   
+
    reg->beg_matches[0] = reg->end_matches[0] = 0;
    buf = reg->buf;
    ebuf = (reg->buf + reg->buf_len) - 2; /* make some room */
@@ -464,15 +511,15 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    if (already_initialized == 0)
      {
 	SLang_init_case_tables ();
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
 	SLmake_lut (Word_Chars, (unsigned char *) "_0-9a-zA-Z\200-\232\240-\245\341-\353", 0);
 #else
 	SLmake_lut (Word_Chars, (unsigned char *) "_0-9a-zA-Z\277-\326\330-\336\340-\366\370-\376", 0);
 #endif
 	already_initialized = 1;
      }
-   
-   i = 1; while (i < 10) 
+
+   i = 1; while (i < 10)
      {
 	reg->beg_matches[i] = -1;
 	reg->end_matches[i] = 0;
@@ -488,25 +535,25 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    else reg->must_match_bol = 0;
 
    *buf = 0;
-   
+
    last_count = count = 0;
    while ((c = *pat++) != 0)
      {
 	if (buf >= ebuf - 3)
 	  {
-	     SLang_doerror ("Pattern large to be compiled.");
+	     SLang_doerror ("Pattern too large to be compiled.");
 	     ERROR;
 	  }
-	
+
 	count++;
 	switch (c)
 	  {
-	   case '$': 
+	   case '$':
 	     if (*pat != 0) goto literal_char;
 	     *buf++ = EOL;
 	     break;
-	     
-	   case '\\': 
+
+	   case '\\':
 	     c = *pat++;
 	     no_osearch = 1;
 	     switch(c)
@@ -516,10 +563,10 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		case 't': c = '\t'; goto literal_char;
 		case 'C': cs = 0; *buf++ = NO_CASE; break;
 		case 'c': cs = 1; *buf++ = YES_CASE; break;
-		case '1': case '2': case '3':  case '4':  case '5':  
-		case '6': case '7': case '8':  case '9': 
+		case '1': case '2': case '3':  case '4':  case '5':
+		case '6': case '7': case '8':  case '9':
 		  c = c - '0';
-		  if ((int) c > nparen / 2) ERROR;
+		  if ((int) c > nparen) ERROR;
 		  last = buf;
 		  *buf++ = NTH_MATCH; *buf++ = c;
 		  break;
@@ -538,17 +585,17 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		  *buf++ = ANY_DIGIT;
 		  min_length++;
 		  break;
-		  
+
 		case '<':
 		  last = NULL;
 		  *buf++ = BOW;
 		  break;
-		  
+
 		case '>':
 		  last = NULL;
 		  *buf++ = EOW;
 		  break;
-		  
+
 		case '{':
 		  if (last == NULL) goto literal_char;
 		  *last |= MANY;
@@ -558,8 +605,8 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		  *buf++ = n;
 
 		  min_length += (n - 1);
-		  
-		  if (*pat == '\\') 
+
+		  if (*pat == '\\')
 		    {
 		       *buf++ = n;
 		    }
@@ -570,7 +617,7 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 			 {
 			    n = 255;
 			 }
-		       else 
+		       else
 			 {
 			    tmppat = convert_digit(pat, &n);
 			    if (tmppat == NULL) ERROR;
@@ -585,17 +632,15 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		  if (*pat != '}') ERROR;
 		  pat++;
 		  break;   /* case '{' */
-		  
-		case '(': 
+
+		case '(':
 		  oparen++;
-		  if ((oparen > '9') || (dparen == 1)) ERROR;
-		  dparen = 1;
-		  nparen++;
+		  if (oparen > 9) ERROR;
 		  *buf++ = OPAREN;
 		  break;
 		case ')':
-		  if (dparen != 1) ERROR;
-		  dparen = 0;
+		  if (oparen == 0) ERROR;
+		  oparen--;
 		  nparen++;
 		  *buf++ = CPAREN;
 		  break;
@@ -610,18 +655,18 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 
 	     *buf = RANGE;
 	     last = buf++;
-	     
+
 	     if (buf + 32 >= ebuf) ERROR;
 
 	     for (i = 0; i < 32; i++) buf[i] = 0;
 	     c = *pat++;
-	     if (c == '^') 
+	     if (c == '^')
 	       {
 		  reverse = 1;
 		  SET_BIT(buf, '\n');
 		  c = *pat++;
 	       }
-	     
+
 	     if (c == ']')
 	       {
 		  SET_BIT(buf, c);
@@ -668,21 +713,21 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 	     buf += 32;
 	     min_length++;
 	     break;
-	     
+
 	   case '.':
 	     last = buf;
 	     *buf++ = ANY;
 	     min_length++;
 	     break;
-	     
-	   case '*': 
+
+	   case '*':
 	     if (last == NULL) goto literal_char;
 	     *last |= STAR;
 	     min_length--;
 	     last = NULL;
 	     break;
-	
-	   case '+': 
+
+	   case '+':
 	     if (last == NULL) goto literal_char;
 	     *last |= LEAST_ONCE;
 	     last = NULL;
@@ -712,14 +757,14 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 		       max_mm = this_max_mm;
 		    }
 	       }
-	     else 
+	     else
 	       {
 		  this_mm_p = buf;
 		  this_max_mm = 1;
 	       }
-	     
+
 	     last_count = count;
-		       
+
 	     last = buf;
 	     *buf++ = LITERAL;
 	     *buf++ = UPPERCASE(c);
@@ -729,14 +774,14 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
    /* Check for ordinary search */
    ebuf = buf;
    buf = reg->buf;
-   
+
    if (no_osearch) ordinary_search = 0;
    else
      {
 	ordinary_search = 1;
 	while (buf < ebuf)
 	  {
-	     if (*buf != LITERAL) 
+	     if (*buf != LITERAL)
 	       {
 		  ordinary_search = 0;
 		  break;
@@ -744,7 +789,7 @@ int SLang_regexp_compile (SLRegexp_Type *reg)
 	     buf += 2;
 	  }
      }
-   
+
    reg->osearch = ordinary_search;
    reg->must_match_str[15] = 0;
    reg->min_length = (min_length > 0) ? (unsigned int) min_length : 0;
@@ -778,12 +823,12 @@ char *SLregexp_quote_string (char *re, char *buf, unsigned int buflen)
 {
    char ch;
    char *b, *bmax;
-   
+
    if (re == NULL) return NULL;
-   
+
    b = buf;
    bmax = buf + buflen;
-   
+
    while (b < bmax)
      {
 	switch (ch = *re++)
@@ -791,7 +836,7 @@ char *SLregexp_quote_string (char *re, char *buf, unsigned int buflen)
 	   case 0:
 	     *b = 0;
 	     return buf;
-	     
+
 	   case '$':
 	   case '\\':
 	   case '[':
@@ -804,7 +849,7 @@ char *SLregexp_quote_string (char *re, char *buf, unsigned int buflen)
 	     *b++ = '\\';
 	    if (b == bmax) break;
 	     /* drop */
-	     
+
 	   default:
 	     *b++ = ch;
 	  }
@@ -820,16 +865,16 @@ int main(int argc, char **argv)
    char *regexp, *file;
    char expbuf[MAX_EXP], buf[512];
    SLRegexp_Type reg;
-   
+
    file = argv[2];
    regexp = argv[1];
-   
+
    if (NULL == (fp = fopen(file, "r")))
      {
 	fprintf(stderr, "File not open\n");
 	return(1);
      }
-   
+
    reg.buf = expbuf;
    reg.buf_len = MAX_EXP;
    reg.pat = regexp;
@@ -841,12 +886,12 @@ int main(int argc, char **argv)
 	  {
 	     if (NULL == strstr(buf, reg.pat)) continue;
 	  }
-	else 
+	else
 	  {
 	     if (reg.must_match && (NULL == strstr(buf, reg.must_match_str))) continue;
 	     if (0 == regexp_match(buf, buf + strlen(buf), &reg)) continue;
 	  }
-	
+
 	fputs(buf, stdout);
      }
    return (0);
