@@ -576,16 +576,8 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
      {
 	key = SLang_do_key (RL_Keymap, (int (*)(void)) rli->getkey);
 
-	if ((key == NULL)
-#ifdef SLKEYMAP_OBSOLETE
-	    || (key->f == NULL)
-#else
-	    || (key->f.f == NULL)
-#endif
-	    )
-	  {
-	     rl_beep ();
-	  }
+	if ((key == NULL) || (key->f.f == NULL))
+	  rl_beep ();
 	else
 	  {
 	     if ((SLang_Last_Key_Char == SLang_RL_EOF_Char)
@@ -594,17 +586,12 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
 	       rl_eof_insert ();
 	     else if (key->type == SLKEY_F_INTRINSIC)
 	       {
-#ifdef SLKEYMAP_OBSOLETE
-		  if (((int (*)(void))(key->f))())
-		    RLupdate (rli);
-#else
 		  if ((key->f.f)())
 		    RLupdate (rli);
 
 		  if ((rli->flags & SL_RLINE_BLINK_MATCH)
 		      && (rli->input_pending != NULL))
 		    blink_match (rli);
-#endif
 	       }
 
 	     if (SLang_Rline_Quit)
@@ -619,13 +606,7 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
 	       }
 	  }
 	if (key != NULL)
-	  {
-#ifdef SLKEYMAP_OBSOLETE
-	     This_RLI->last_fun = key->f;
-#else
-	     This_RLI->last_fun = key->f.f;
-#endif
-	  }
+	  This_RLI->last_fun = key->f.f;
      }
 }
 
@@ -737,62 +718,30 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	for (ch = ' '; ch < 256; ch++)
 	  {
 	     simple[0] = (char) ch;
-# ifdef SLKEYMAP_OBSOLETE
-	     SLang_define_key1(simple, (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-# else
 	     SLkm_define_key (simple, (FVOID_STAR) rl_self_insert, RL_Keymap);
-# endif
 	  }
 #else
 	ch = ' ';
 	while (1)
 	  {
 	     simple[0] = (char) ch;
-# ifdef SLKEYMAP_OBSOLETE
-	     SLang_define_key1(simple, (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-# else
 	     SLkm_define_key (simple, (FVOID_STAR) rl_self_insert, RL_Keymap);
-# endif
 	     ch = ch + 1;
 	     if (ch == 256) break;
 	  }
 #endif				       /* NOT __DECC */
 
 	simple[0] = SLang_Abort_Char;
-#ifdef SLKEYMAP_OBSOLETE
-	SLang_define_key1(simple, (VOID_STAR) rl_abort, SLKEY_F_INTRINSIC, RL_Keymap);
-#else
 	SLkm_define_key (simple, (FVOID_STAR) rl_abort, RL_Keymap);
-#endif
 	simple[0] = SLang_RL_EOF_Char;
-#ifdef SLKEYMAP_OBSOLETE
-	SLang_define_key1(simple, (VOID_STAR) rl_eof_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-#else
 	SLkm_define_key (simple, (FVOID_STAR) rl_eof_insert, RL_Keymap);
-#endif
 
 #ifndef IBMPC_SYSTEM
-# ifdef SLKEYMAP_OBSOLETE
-	SLang_define_key1 ("^[[A", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[B", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[C", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^[[D", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
-# else
 	SLkm_define_key  ("^[[A", (FVOID_STAR) rl_prev_line, RL_Keymap);
 	SLkm_define_key  ("^[[B", (FVOID_STAR) rl_next_line, RL_Keymap);
 	SLkm_define_key  ("^[[C", (FVOID_STAR) rl_right, RL_Keymap);
 	SLkm_define_key  ("^[[D", (FVOID_STAR) rl_left, RL_Keymap);
-# endif
 #else
-# ifdef SLKEYMAP_OBSOLETE
-	SLang_define_key1 ("^@H", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@P", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@M", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@K", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@S", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@O", (VOID_STAR) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^@G", (VOID_STAR) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
-# else
 	SLkm_define_key  ("^@H", (FVOID_STAR) rl_prev_line, RL_Keymap);
 	SLkm_define_key  ("^@P", (FVOID_STAR) rl_next_line, RL_Keymap);
 	SLkm_define_key  ("^@M", (FVOID_STAR) rl_right, RL_Keymap);
@@ -800,29 +749,7 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	SLkm_define_key  ("^@S", (FVOID_STAR) rl_del, RL_Keymap);
 	SLkm_define_key  ("^@O", (FVOID_STAR) rl_eol, RL_Keymap);
 	SLkm_define_key  ("^@G", (FVOID_STAR) rl_bol, RL_Keymap);
-# endif
 #endif
-#ifdef SLKEYMAP_OBSOLETE
-	SLang_define_key1 ("^E", (VOID_STAR) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^I", (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^A", (VOID_STAR) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("\r", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("\n", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^K", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^L", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^V", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^D", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^F", (VOID_STAR) rl_right, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^B", (VOID_STAR) rl_left, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^?", (VOID_STAR) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^H", (VOID_STAR) rl_bdel, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^P", (VOID_STAR) rl_prev_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^N", (VOID_STAR) rl_next_line, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^R", (VOID_STAR) rl_redraw, SLKEY_F_INTRINSIC, RL_Keymap);
-
-	SLang_define_key1 ("`", (VOID_STAR) rl_quote_insert, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("\033\\", (VOID_STAR) rl_trim, SLKEY_F_INTRINSIC, RL_Keymap);
-#else
 	SLkm_define_key  ("^C", (FVOID_STAR) rl_abort, RL_Keymap);
 	SLkm_define_key  ("^E", (FVOID_STAR) rl_eol, RL_Keymap);
 	SLkm_define_key  ("^G", (FVOID_STAR) rl_abort, RL_Keymap);
@@ -843,7 +770,6 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	SLkm_define_key  ("^R", (FVOID_STAR) rl_redraw, RL_Keymap);
 	SLkm_define_key  ("`", (FVOID_STAR) rl_quote_insert, RL_Keymap);
 	SLkm_define_key  ("\033\\", (FVOID_STAR) rl_trim, RL_Keymap);
-#endif
 	if (SLang_Error) return -1;
      }
 
