@@ -1,16 +1,17 @@
-/* Copyright (c) 1992, 1995 John E. Davis
- * All rights reserved.
- * 
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of the S-Lang library.
+ *
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
 #include "config.h"
+#include "sl-feat.h"
 
 #include <stdio.h>
 #include <string.h>
 
-
 #include "slang.h"
+#include "_slang.h"
 
 #ifdef upcase
 #undef upcase
@@ -18,16 +19,16 @@
 
 #define upcase(ch) (cs ? ch : UPPER_CASE(ch))
 
-static unsigned char *search_forward (register unsigned char *beg, 
-				      unsigned char *end, 
-				      unsigned char *key, 
+static unsigned char *search_forward (register unsigned char *beg,
+				      unsigned char *end,
+				      unsigned char *key,
 				      register int key_len, int cs, int *ind)
 {
    register unsigned char char1;
    unsigned char *pos;
    int j, str_len;
    register unsigned char ch;
-   register int db;   
+   register int db;
 
    str_len = (int) (end - beg);
    if (str_len < key_len) return (NULL);
@@ -48,20 +49,20 @@ static unsigned char *search_forward (register unsigned char *beg,
 	  {
 	     ch = *beg;
 	     db = ind[(unsigned char) ch];
-	     if ((db < key_len) && 
+	     if ((db < key_len) &&
 		 (UPPER_CASE(ch) == char1)) break;
 	     beg += db; /* ind[(unsigned char) ch]; */
 	  }
-	
+
 	if (beg >= end) return(NULL);
-	
+
 	pos = beg - (key_len - 1);
 	for (j = 0; j < key_len; j++)
 	  {
 	     ch = upcase(pos[j]);
 	     if (ch != (unsigned char) key[j]) break;
 	  }
-	
+
 	if (j == key_len) return(pos);
 	beg += 1;
      }
@@ -76,7 +77,7 @@ static unsigned char *search_backward (unsigned char *beg,unsigned char *end,
 
     str_len = (int) (end - beg);
     if (str_len < key_len) return (NULL);
-   
+
    /*  end -= (key_len - 1); */
    end -= key_len;
 
@@ -87,7 +88,7 @@ static unsigned char *search_backward (unsigned char *beg,unsigned char *end,
 	 while ((beg <= end) && (ch = *end, ch = upcase(ch), ch != char1))
 	   {
 	      ofs = ind[(unsigned char) ch];
-#ifdef msdos
+#ifdef __MSDOS__
 	      /* This is needed for msdos segment wrapping problems */
 	      if (beg + ofs > end) return(NULL);
 #endif
@@ -104,7 +105,7 @@ static unsigned char *search_backward (unsigned char *beg,unsigned char *end,
       }
 }
 
-unsigned char *SLsearch (unsigned char *pmin, unsigned char *pmax, 
+unsigned char *SLsearch (unsigned char *pmin, unsigned char *pmax,
 			 SLsearch_Type *st)
 {
    if (st->dir > 0) return search_forward (pmin, pmax, st->key,
@@ -122,18 +123,18 @@ int SLsearch_init (char *str, int dir, int cs, SLsearch_Type *st)
    unsigned char *w, *work = st->key;
    register int *indp, *indpm;
    int *ind = st->ind;
-   
+
    if (max >= (int) sizeof (st->key))
      {
 	SLang_doerror ("Search string too long.");
 	return -1;
      }
-   
+
    st->dir = dir; st->cs = cs;
-   
+
    if (!Case_Tables_Ok) SLang_init_case_tables ();
-   
-   if (dir > 0) 
+
+   if (dir > 0)
      {
 	w = work;
      }
@@ -143,85 +144,92 @@ int SLsearch_init (char *str, int dir, int cs, SLsearch_Type *st)
 	str = str + maxi;
 	w = work + maxi;
      }
-   
+
    /* for (i = 0; i < 256; i++) ind[i] = max; */
-   indp = ind; indpm = ind + 256; while (indp < indpm) *indp++ = max;
-   
-   i = 0;
-   while (i++ < max)
+   indp = ind; indpm = ind + 256; 
+   while (indp < indpm) 
      {
+	*indp++ = max;
+	*indp++ = max;
+	*indp++ = max;
+	*indp++ = max;
+     }
+
+   i = 0;
+   if (cs) while (i < max)
+     {
+	i++;
 	maxi = max - i;
-	if (cs)
-	  {
-	     *w = *str;
-	     ind[(unsigned char) *str] = maxi;
-	  }
-	else
-	  {
-	     *w = UPPER_CASE(*str);
-	     ind[(unsigned char) *w] = maxi;
-	     ind[(unsigned char) LOWER_CASE(*str)] = maxi;
-	  }
+	*w = *str;
+	ind[(unsigned char) *str] = maxi;
 	str += dir; w += dir;
      }
+   else while (i < max)
+     {
+	i++;
+	maxi = max - i;
+	*w = UPPER_CASE(*str);
+	ind[(unsigned char) *w] = maxi;
+	ind[(unsigned char) LOWER_CASE(*str)] = maxi;
+	str += dir; w += dir;
+     }
+
    work[max] = 0;
    st->key_len = max;
    return max;
 }
 
-
 /* 8bit clean upper and lowercase macros */
-unsigned char Chg_LCase_Lut[256];
-unsigned char Chg_UCase_Lut[256];
+unsigned char _SLChg_LCase_Lut[256];
+unsigned char _SLChg_UCase_Lut[256];
 
 void SLang_define_case(int *u, int *l)
 {
    unsigned char up = (unsigned char) *u, dn = (unsigned char) *l;
-   
-   Chg_LCase_Lut[up] = dn;
-   Chg_UCase_Lut[dn] = up;
-}
 
+   _SLChg_LCase_Lut[up] = dn;
+   _SLChg_UCase_Lut[dn] = up;
+}
 
 void SLang_init_case_tables (void)
 {
    int i, j;
    if (Case_Tables_Ok) return;
-   
-   for (i = 0; i < 256; i++) 
+
+   for (i = 0; i < 256; i++)
      {
-	Chg_UCase_Lut[i] = i;
-	Chg_LCase_Lut[i] = i;
+	_SLChg_UCase_Lut[i] = i;
+	_SLChg_LCase_Lut[i] = i;
      }
-   
-   for (i = 'A'; i <= 'Z'; i++) 
+
+   for (i = 'A'; i <= 'Z'; i++)
      {
 	j = i + 32;
-	Chg_UCase_Lut[j] = i;
-	Chg_LCase_Lut[i] = j;
+	_SLChg_UCase_Lut[j] = i;
+	_SLChg_LCase_Lut[i] = j;
      }
-#ifdef msdos
+#ifdef PC_SYSTEM
    /* Initialize for DOS code page 437. */
-   Chg_UCase_Lut[135] = 128; Chg_LCase_Lut[128] = 135;
-   Chg_UCase_Lut[132] = 142; Chg_LCase_Lut[142] = 132;
-   Chg_UCase_Lut[134] = 143; Chg_LCase_Lut[143] = 134;
-   Chg_UCase_Lut[130] = 144; Chg_LCase_Lut[144] = 130;
-   Chg_UCase_Lut[145] = 146; Chg_LCase_Lut[146] = 145;
-   Chg_UCase_Lut[148] = 153; Chg_LCase_Lut[153] = 148;
-   Chg_UCase_Lut[129] = 154; Chg_LCase_Lut[154] = 129;
-   Chg_UCase_Lut[164] = 165; Chg_LCase_Lut[165] = 164;
+   _SLChg_UCase_Lut[135] = 128; _SLChg_LCase_Lut[128] = 135;
+   _SLChg_UCase_Lut[132] = 142; _SLChg_LCase_Lut[142] = 132;
+   _SLChg_UCase_Lut[134] = 143; _SLChg_LCase_Lut[143] = 134;
+   _SLChg_UCase_Lut[130] = 144; _SLChg_LCase_Lut[144] = 130;
+   _SLChg_UCase_Lut[145] = 146; _SLChg_LCase_Lut[146] = 145;
+   _SLChg_UCase_Lut[148] = 153; _SLChg_LCase_Lut[153] = 148;
+   _SLChg_UCase_Lut[129] = 154; _SLChg_LCase_Lut[154] = 129;
+   _SLChg_UCase_Lut[164] = 165; _SLChg_LCase_Lut[165] = 164;
 #else
    /* ISO Latin */
-   for (i = 192; i <= 221; i++) 
+   for (i = 192; i <= 221; i++)
      {
 	j = i + 32;
-	Chg_UCase_Lut[j] = i;
-	Chg_LCase_Lut[i] = j;
+	_SLChg_UCase_Lut[j] = i;
+	_SLChg_LCase_Lut[i] = j;
      }
-   Chg_UCase_Lut[215] = 215; Chg_LCase_Lut[215] = 215;
-   Chg_UCase_Lut[223] = 223; Chg_LCase_Lut[223] = 223;
-   Chg_UCase_Lut[247] = 247; Chg_LCase_Lut[247] = 247;
-   Chg_UCase_Lut[255] = 255; Chg_LCase_Lut[255] = 255;
+   _SLChg_UCase_Lut[215] = 215; _SLChg_LCase_Lut[215] = 215;
+   _SLChg_UCase_Lut[223] = 223; _SLChg_LCase_Lut[223] = 223;
+   _SLChg_UCase_Lut[247] = 247; _SLChg_LCase_Lut[247] = 247;
+   _SLChg_UCase_Lut[255] = 255; _SLChg_LCase_Lut[255] = 255;
 #endif
    Case_Tables_Ok = 1;
 }
