@@ -84,14 +84,17 @@ static char *map_token_to_string (_SLang_Token_Type *tok)
    return s;
 }
 
-static char *make_line_file_error (_SLang_Token_Type *tok, char *dsc, char *buf, int line, char *file)
+static char *make_line_file_error (char *buf, unsigned int buflen, 
+				   _SLang_Token_Type *tok, char *dsc, int line, char *file)
 {
 #if _SLANG_HAS_DEBUG_CODE
    if (tok != NULL) line = tok->line_number;
 #endif
-   if (file == NULL) file = "??";   
-   sprintf (buf, "%s: found '%s', line %d, file: %s",
-	      dsc, map_token_to_string (tok), line, file);
+   if (file == NULL) file = "??";
+   
+   (void) _SLsnprintf (buf, buflen, "%s: found '%s', line %d, file: %s",
+		       dsc, map_token_to_string (tok), line, file);
+
    return buf;
 }
 
@@ -102,7 +105,7 @@ void _SLparse_error(char *str, _SLang_Token_Type *tok, int flag)
    if (str == NULL)
      str = "Parse Error";
 
-   make_line_file_error (tok, str, buf, LLT->line_num, (char *) LLT->name);
+   make_line_file_error (buf, sizeof (buf), tok, str, LLT->line_num, (char *) LLT->name);
 
    if ((flag == 0) && SLang_Error)
      return;
@@ -339,6 +342,9 @@ static int get_number_token (_SLang_Token_Type *tok, unsigned char *s, unsigned
 	     /* It must be hex */
 	     do
 	       {
+		  if (len == (MAX_TOKEN_LEN - 1))
+		    goto too_long_return_error;
+
 		  s[len++] = ch;
 		  ch = prep_get_char ();
 		  type = CHAR_CLASS (ch);
@@ -346,6 +352,8 @@ static int get_number_token (_SLang_Token_Type *tok, unsigned char *s, unsigned
 	     while ((type == DIGIT_CHAR) || (type == ALPHA_CHAR));
 	     break;
 	  }
+	if (len == (MAX_TOKEN_LEN - 1))
+	  goto too_long_return_error;
 	s [len++] = ch;
      }
 
@@ -353,16 +361,22 @@ static int get_number_token (_SLang_Token_Type *tok, unsigned char *s, unsigned
 
    if ((ch == 'e') || (ch == 'E'))
      {
+	if (len == (MAX_TOKEN_LEN - 1))
+	  goto too_long_return_error;
 	s[len++] = ch;
 	ch = prep_get_char ();
 	if ((ch == '+') || (ch == '-'))
 	  {
+	     if (len == (MAX_TOKEN_LEN - 1))
+	       goto too_long_return_error;
 	     s[len++] = ch;
 	     ch = prep_get_char ();
 	  }
 
 	while (DIGIT_CHAR == (type = CHAR_CLASS(ch)))
 	  {
+	     if (len == (MAX_TOKEN_LEN - 1))
+	       goto too_long_return_error;
 	     s[len++] = ch;
 	     ch = prep_get_char ();
 	  }
@@ -370,6 +384,8 @@ static int get_number_token (_SLang_Token_Type *tok, unsigned char *s, unsigned
 
    while (ALPHA_CHAR == type)
      {
+	if (len == (MAX_TOKEN_LEN - 1))
+	  goto too_long_return_error;
 	s[len++] = ch;
 	ch = prep_get_char ();
 	type = CHAR_CLASS(ch);
@@ -400,6 +416,10 @@ static int get_number_token (_SLang_Token_Type *tok, unsigned char *s, unsigned
    tok->v.s_val = (char *) s;
    _SLparse_error ("Not a number", tok, 0);
    return (tok->type = EOF_TOKEN);
+   
+   too_long_return_error:
+   _SLparse_error ("Number too long for buffer", NULL, 0);
+   return (tok->type == EOF_TOKEN);
 }
 
 /* The order here must match the order in _slang.h */
@@ -411,7 +431,7 @@ Binary_Token_Name_Type;
 
 static Binary_Token_Name_Type Binary_Token_Names [] =
 {
-   {"^"}, {"+"}, {"-"}, {"*"}, {"/"}, {"<"}, {"<="}, {">"}, {">="}, {"=="},
+   {"+"}, {"-"}, {"*"}, {"/"}, {"<"}, {"<="}, {">"}, {">="}, {"=="},
    {"!="}, {"and"}, {"or"}, {"mod"}, {"&"}, {"shl"}, {"shr"}, {"xor"},{"|"}
 };
 
